package ava.parser

sealed abstract class Keyword(val value: String)

object Keyword:

  case object Alias     extends Keyword("alias")
  case object Case      extends Keyword("case")
  case object Class     extends Keyword("class")
  case object Const     extends Keyword("const")
  case object Defn      extends Keyword("defn")
  case object Do        extends Keyword("do")
  case object Else      extends Keyword("else")
  case object End       extends Keyword("end")
  case object Enum      extends Keyword("enum")
  case object Export    extends Keyword("export")
  case object False     extends Keyword("false")
  case object Fn        extends Keyword("fn")
  case object Given     extends Keyword("given")
  case object If        extends Keyword("if")
  case object Import    extends Keyword("import")
  case object Infix     extends Keyword("infix")
  case object Lambda    extends Keyword("λ")
  case object Let       extends Keyword("let")
  case object Match     extends Keyword("match")
  case object Mut       extends Keyword("mut")
  case object Namespace extends Keyword("namespace")
  case object Object    extends Keyword("object")
  case object Private   extends Keyword("private")
  case object Record    extends Keyword("record")
  case object Return    extends Keyword("return")
  case object Then      extends Keyword("then")
  case object True      extends Keyword("true")
  case object Type      extends Keyword("type")

  val All: List[Keyword] = List(
    Alias,
    Case,
    Class,
    Const,
    Defn,
    Do,
    Else,
    End,
    Enum,
    Export,
    False,
    Fn,
    Given,
    If,
    Import,
    Infix,
    Lambda,
    Let,
    Match,
    Mut,
    Namespace,
    Object,
    Private,
    Record,
    Return,
    Then,
    True,
    Type
  )

  def isKeyword(candidate: String): Boolean =
    All.contains(candidate)

end Keyword
