package ava.parser

sealed abstract class Operator(val value: String)

object Operator:

  case object Hole        extends Operator("???")
  case object ImportSplat extends Operator("*")
  case object Member      extends Operator(".")
  case object Union       extends Operator("|")
  case object ListPrepend extends Operator(":-")
  case object BindType    extends Operator(":")
  case object BindValue   extends Operator(":=")
  case object FnReturn    extends Operator("->")
  case object BindDo      extends Operator("<-")
  case object ClassMember extends Operator("::")
  case object Case        extends Operator("=>")

  val All: List[Operator] = List(
    Hole,
    ImportSplat,
    Member,
    Union,
    ListPrepend,
    BindType,
    BindValue,
    FnReturn,
    BindDo,
    ClassMember,
    Case
  )

  /** List of unambiguous operators that can always be trivially parsed. Other
    * operators are either specific tokens, or a combination of tokens.
    */
  val Trivial: List[Operator] = List(
    Hole,
    ImportSplat,
    Union,
    FnReturn,
    BindDo,
    Case
  )

end Operator
