package ava.parser

class RingTests extends munit.FunSuite:

  test("should fail to instantiate with <= 0 capacity") {
    interceptMessage[IllegalArgumentException](
      "Rings may only be constructed with capacity > 0"
    ) {
      val _ = Ring[Int](-1, 0)
    }
    interceptMessage[IllegalArgumentException](
      "Rings may only be constructed with capacity > 0"
    ) {
      val _ = Ring[Int](0, 0)
    }
  }

  test("should return the buffer size independent of capacity") {
    val capacity: Int = 4
    val ring          = Ring[Int](capacity, 0)
    assertEquals(ring.size(), 0)
    assertEquals(ring.capacity, capacity)
    assertEquals(ring.unit, 0)
    var item = 1
    while item <= capacity do
      ring.push(item)
      assertEquals(ring.size(), item)
      item = item + 1
  }

  test("should fail to retrieve the newest element if not initialized") {
    interceptMessage[IllegalStateException](
      "This ring has not been initialized with any data."
    ) {
      val ring = Ring[Int](1, 0)
      val _    = ring.newest()
    }
  }

  test("should return the newest element of the ring") {
    val capacity: Int = 4
    val unit: Int     = 0
    val ring          = Ring[Int](capacity, unit)
    var item          = 1
    while item <= capacity do
      ring.push(item)
      assertEquals(ring.newest(), item)
      item = item + 1
  }

  test("should respect ordering across cycles") {
    val capacity: Int = 4
    val unit: Int     = 0
    val ring          = Ring[Int](capacity, unit)
    var item          = 1
    while item <= capacity do
      ring.push(item)
      item = item + 1

    assertEquals(ring.size(), capacity)
    val expected = Array[Int](5, 4, 3, 2)
    item = 1
    while item <= capacity + 1 do
      ring.push(item)
      item = item + 1
      assertEquals(ring.size(), capacity)
    val extract = ring.newestToOldest()
    assert(extract.sameElements(expected))
  }
